@@ -1,9 +1,8 @@
 from unittest import TestCase
 
-from dogpile.cache import util
-
 
 class MakoTest(TestCase):
+
     """ Test entry point for Mako
     """
 
@@ -12,6 +11,7 @@ class MakoTest(TestCase):
 
         # if the entrypoint isn't there, just pass, as the tests can be run
         # without any setuptools install
-        for impl in pkg_resources.iter_entry_points("mako.cache", "dogpile.cache"):
+        for impl in pkg_resources.iter_entry_points(
+            "mako.cache", "dogpile.cache"
+        ):
             impl.load()
-
