@@ -1,49 +1,87 @@
-from ._fixtures import _GenericBackendTest, _GenericMutexTest
-from . import eq_
-from unittest import TestCase
+import os
+import ssl
 from threading import Thread
 import time
-import pytest
-from dogpile.util import compat
-import os
+from unittest import TestCase
 import weakref
 
+import pytest
 
-MEMCACHED_PORT = os.getenv('DOGPILE_MEMCACHED_PORT', '11211')
+from dogpile.cache.backends.memcached import GenericMemcachedBackend
+from dogpile.cache.backends.memcached import MemcachedBackend
+from dogpile.cache.backends.memcached import PylibmcBackend
+from . import eq_
+from ._fixtures import _GenericBackendTest
+from ._fixtures import _GenericMutexTest
+
+
+MEMCACHED_PORT = os.getenv("DOGPILE_MEMCACHED_PORT", "11211")
 MEMCACHED_URL = "127.0.0.1:%s" % MEMCACHED_PORT
+expect_memcached_running = bool(os.getenv("DOGPILE_MEMCACHED_PORT"))
+
+TLS_CONTEXT = ssl.create_default_context(cafile="tests/tls/ca-root.crt")
+TLS_MEMCACHED_PORT = os.getenv("DOGPILE_TLS_MEMCACHED_PORT", "11212")
+TLS_MEMCACHED_URL = "localhost:%s" % TLS_MEMCACHED_PORT
+expect_tls_memcached_running = bool(os.getenv("DOGPILE_TLS_MEMCACHED_PORT"))
 
 LOCK_TIMEOUT = 1
 
 
 class _TestMemcachedConn(object):
-
     @classmethod
     def _check_backend_available(cls, backend):
         try:
             client = backend._create_client()
             client.set("x", "y")
             assert client.get("x") == "y"
-        except:
-            pytest.skip(
-                "memcached is not running or "
-                "otherwise not functioning correctly")
+        except Exception:
+            if not expect_memcached_running:
+                pytest.skip(
+                    "memcached is not running or "
+                    "otherwise not functioning correctly"
+                )
+            else:
+                raise
+
+
+class _TestTLSMemcachedConn(object):
+    @classmethod
+    def _check_backend_available(cls, backend):
+        try:
+            client = backend._create_client()
+            client.set("x", "y")
+            assert client.get("x") == "y"
+        except Exception:
+            if not expect_tls_memcached_running:
+                pytest.skip(
+                    "TLS memcached is not running or "
+                    "otherwise not functioning correctly"
+                )
+            else:
+                raise
 
 
 class _NonDistributedMemcachedTest(_TestMemcachedConn, _GenericBackendTest):
-    region_args = {
-        "key_mangler": lambda x: x.replace(" ", "_")
-    }
+    region_args = {"key_mangler": lambda x: x.replace(" ", "_")}
+    config_args = {"arguments": {"url": MEMCACHED_URL}}
+
+
+class _NonDistributedTLSMemcachedTest(
+    _TestTLSMemcachedConn, _GenericBackendTest
+):
+    region_args = {"key_mangler": lambda x: x.replace(" ", "_")}
     config_args = {
         "arguments": {
-            "url": MEMCACHED_URL
+            "url": TLS_MEMCACHED_URL,
+            "tls_context": TLS_CONTEXT,
         }
     }
 
 
-class _DistributedMemcachedWithTimeoutTest(_TestMemcachedConn, _GenericBackendTest):
-    region_args = {
-        "key_mangler": lambda x: x.replace(" ", "_")
-    }
+class _DistributedMemcachedWithTimeoutTest(
+    _TestMemcachedConn, _GenericBackendTest
+):
+    region_args = {"key_mangler": lambda x: x.replace(" ", "_")}
     config_args = {
         "arguments": {
             "url": MEMCACHED_URL,
@@ -54,27 +92,21 @@ class _DistributedMemcachedWithTimeoutTest(_TestMemcachedConn, _GenericBackendTe
 
 
 class _DistributedMemcachedTest(_TestMemcachedConn, _GenericBackendTest):
-    region_args = {
-        "key_mangler": lambda x: x.replace(" ", "_")
-    }
+    region_args = {"key_mangler": lambda x: x.replace(" ", "_")}
     config_args = {
-        "arguments": {
-            "url": MEMCACHED_URL,
-            "distributed_lock": True,
-        }
+        "arguments": {"url": MEMCACHED_URL, "distributed_lock": True}
     }
 
 
 class _DistributedMemcachedMutexTest(_TestMemcachedConn, _GenericMutexTest):
     config_args = {
-        "arguments": {
-            "url": MEMCACHED_URL,
-            "distributed_lock": True
-        }
+        "arguments": {"url": MEMCACHED_URL, "distributed_lock": True}
     }
 
 
-class _DistributedMemcachedMutexWithTimeoutTest(_TestMemcachedConn, _GenericMutexTest):
+class _DistributedMemcachedMutexWithTimeoutTest(
+    _TestMemcachedConn, _GenericMutexTest
+):
     config_args = {
         "arguments": {
             "url": MEMCACHED_URL,
@@ -96,41 +128,31 @@ class PylibmcDistributedMutexTest(_DistributedMemcachedMutexTest):
     backend = "dogpile.cache.pylibmc"
 
 
-class BMemcachedSkips(object):
-    def test_threaded_dogpile(self):
-        pytest.skip("bmemcached is too unreliable here")
-
-    def test_threaded_get_multi(self):
-        pytest.skip("bmemcached is too unreliable here")
-
-    def test_mutex_threaded_dogpile(self):
-        pytest.skip("bmemcached is too unreliable here")
-
-    def test_mutex_threaded(self):
-        pytest.skip("bmemcached is too unreliable here")
-
-
-class BMemcachedTest(BMemcachedSkips, _NonDistributedMemcachedTest):
+class BMemcachedTest(_NonDistributedMemcachedTest):
     backend = "dogpile.cache.bmemcached"
 
 
 class BMemcachedDistributedWithTimeoutTest(
-        BMemcachedSkips, _DistributedMemcachedWithTimeoutTest):
+    _DistributedMemcachedWithTimeoutTest
+):
     backend = "dogpile.cache.bmemcached"
 
 
-class BMemcachedDistributedTest(
-        BMemcachedSkips, _DistributedMemcachedTest):
+class BMemcachedTLSTest(_NonDistributedTLSMemcachedTest):
     backend = "dogpile.cache.bmemcached"
 
 
-class BMemcachedDistributedMutexTest(
-        BMemcachedSkips, _DistributedMemcachedMutexTest):
+class BMemcachedDistributedTest(_DistributedMemcachedTest):
+    backend = "dogpile.cache.bmemcached"
+
+
+class BMemcachedDistributedMutexTest(_DistributedMemcachedMutexTest):
     backend = "dogpile.cache.bmemcached"
 
 
 class BMemcachedDistributedMutexWithTimeoutTest(
-        BMemcachedSkips, _DistributedMemcachedMutexWithTimeoutTest):
+    _DistributedMemcachedMutexWithTimeoutTest
+):
     backend = "dogpile.cache.bmemcached"
 
 
@@ -146,13 +168,7 @@ class MemcachedDistributedMutexTest(_DistributedMemcachedMutexTest):
     backend = "dogpile.cache.memcached"
 
 
-from dogpile.cache.backends.memcached import GenericMemcachedBackend
-from dogpile.cache.backends.memcached import PylibmcBackend
-from dogpile.cache.backends.memcached import MemcachedBackend
-
-
 class MockGenericMemcachedBackend(GenericMemcachedBackend):
-
     def _imports(self):
         pass
 
@@ -161,7 +177,6 @@ class MockGenericMemcachedBackend(GenericMemcachedBackend):
 
 
 class MockMemcacheBackend(MemcachedBackend):
-
     def _imports(self):
         pass
 
@@ -170,15 +185,13 @@ class MockMemcacheBackend(MemcachedBackend):
 
 
 class MockPylibmcBackend(PylibmcBackend):
-
     def _imports(self):
         pass
 
     def _create_client(self):
-        return MockClient(self.url,
-                          binary=self.binary,
-                          behaviors=self.behaviors
-                          )
+        return MockClient(
+            self.url, binary=self.binary, behaviors=self.behaviors
+        )
 
 
 class MockClient(object):
@@ -211,61 +224,64 @@ class MockClient(object):
 
 
 class PylibmcArgsTest(TestCase):
-
     def test_binary_flag(self):
-        backend = MockPylibmcBackend(arguments={'url': 'foo', 'binary': True})
+        backend = MockPylibmcBackend(arguments={"url": "foo", "binary": True})
         eq_(backend._create_client().kw["binary"], True)
 
     def test_url_list(self):
-        backend = MockPylibmcBackend(arguments={'url': ["a", "b", "c"]})
+        backend = MockPylibmcBackend(arguments={"url": ["a", "b", "c"]})
         eq_(backend._create_client().arg[0], ["a", "b", "c"])
 
     def test_url_scalar(self):
-        backend = MockPylibmcBackend(arguments={'url': "foo"})
+        backend = MockPylibmcBackend(arguments={"url": "foo"})
         eq_(backend._create_client().arg[0], ["foo"])
 
     def test_behaviors(self):
-        backend = MockPylibmcBackend(arguments={'url': "foo",
-                                                "behaviors": {"q": "p"}})
+        backend = MockPylibmcBackend(
+            arguments={"url": "foo", "behaviors": {"q": "p"}}
+        )
         eq_(backend._create_client().kw["behaviors"], {"q": "p"})
 
     def test_set_time(self):
-        backend = MockPylibmcBackend(arguments={'url': "foo",
-                                                "memcached_expire_time": 20})
+        backend = MockPylibmcBackend(
+            arguments={"url": "foo", "memcached_expire_time": 20}
+        )
         backend.set("foo", "bar")
         eq_(backend._clients.memcached.canary, [{"time": 20}])
 
     def test_set_min_compress_len(self):
-        backend = MockPylibmcBackend(arguments={'url': "foo",
-                                                "min_compress_len": 20})
+        backend = MockPylibmcBackend(
+            arguments={"url": "foo", "min_compress_len": 20}
+        )
         backend.set("foo", "bar")
         eq_(backend._clients.memcached.canary, [{"min_compress_len": 20}])
 
     def test_no_set_args(self):
-        backend = MockPylibmcBackend(arguments={'url': "foo"})
+        backend = MockPylibmcBackend(arguments={"url": "foo"})
         backend.set("foo", "bar")
         eq_(backend._clients.memcached.canary, [{}])
 
 
 class MemcachedArgstest(TestCase):
-
     def test_set_time(self):
-        backend = MockMemcacheBackend(arguments={'url': "foo",
-                                                 "memcached_expire_time": 20})
+        backend = MockMemcacheBackend(
+            arguments={"url": "foo", "memcached_expire_time": 20}
+        )
         backend.set("foo", "bar")
         eq_(backend._clients.memcached.canary, [{"time": 20}])
 
     def test_set_min_compress_len(self):
-        backend = MockMemcacheBackend(arguments={'url': "foo",
-                                                 "min_compress_len": 20})
+        backend = MockMemcacheBackend(
+            arguments={"url": "foo", "min_compress_len": 20}
+        )
         backend.set("foo", "bar")
         eq_(backend._clients.memcached.canary, [{"min_compress_len": 20}])
 
 
 class LocalThreadTest(TestCase):
-
     def setUp(self):
         import gc
+
         gc.collect()
         eq_(MockClient.number_of_clients(), 0)
 
@@ -279,21 +295,26 @@ class LocalThreadTest(TestCase):
         self._test_client_cleanup(10)
 
     def _test_client_cleanup(self, count):
-        backend = MockGenericMemcachedBackend(arguments={'url': 'foo'})
+        backend = MockGenericMemcachedBackend(arguments={"url": "foo"})
         canary = []
 
-        def f():
+        flag = [False]
+
+        def f(delay):
             backend._clients.memcached
             canary.append(MockClient.number_of_clients())
-            time.sleep(.05)
+            while not flag[0]:
+                time.sleep(0.02)
 
-        threads = [Thread(target=f) for i in range(count)]
+        threads = [Thread(target=f, args=(count - i,)) for i in range(count)]
         for t in threads:
             t.start()
+        flag[0] = True
         for t in threads:
             t.join()
         eq_(canary, [i + 1 for i in range(count)])
 
         import gc
+
         gc.collect()
         eq_(MockClient.number_of_clients(), 0)
